package com.brilliance.isc.funds.fxtssb.service;

import com.brilliance.isc.bo.Bch;
import com.brilliance.isc.bo.Pta;
import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.glemod.service.GlemodService;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.FxtssbStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtssb.check.FxtssbValidator;
import com.brilliance.isc.funds.vo.funds.FxtssbStoreVo;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.IStream;
import com.brilliance.mda.runtime.mda.impl.Argument;
import com.brilliance.mda.runtime.mda.impl.StreamImpl;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Streams;
import com.brilliance.mda.runtime.mda.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: FxtssbTransactionServiceImpl
 * @description: 即期结汇 Fxtssb 业务处理
 * @author: huangshunlin
 * @create: 2024-10-08 15:35
 */
@Service("fxtssb.transaction")
public class FxtssbTransactionServiceImpl extends AbstractTransactionService<FxtssbStoreBo> {

    @Resource
    private FxtssbValidator fxtssbValidator;

    @Resource
    private XrtmodService xrtmodService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private FxToolComponent fxToolComponent;


    @Resource
    private TransactionHelpService transactionHelpService;


    @Resource
    private SettleContext settleContext;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private TrnmodService trnmodService;

    @Resource
    private GlemodService glemodService;

    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
    }


    @Override
    protected void txnSave(FxtssbStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    public void ruleMtabutSavN500(FxtssbStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        Fxdgrp fxdgrp = bo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        // 1. Create (rsp. update) Contract (incl. Texts)
        Argument<String> pathBox = new Argument<String>("", fxdgrp.getRec().getAccmng());
        Argument<String> pathBox1 = new Argument<String>("", fxdgrp.getRec().getHdbch());
        sysmodService.getPtyAccMng(fxdgrp.getApl().getPta().getPtyinr(), "", "", "", pathBox, pathBox1);

        fxdgrp.getRec().setAccmng(pathBox.value);
        fxdgrp.getRec().setHdbch(pathBox1.value);
        fxToolComponent.sav(fxdgrp, BigDecimal.ZERO);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract(trnmodVo);

        cbsSav(bo, trnmodVo);
        // save documents

    }

    @Override
    protected Map<String, String> txnCheck(FxtssbStoreBo transactionVo) {
        return fxtssbValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtssbStoreBo transactionVo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtssbStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtssbStoreBo init(FxtssbStoreBo fxtssbStoreBo) {
        fxtssbStoreBo.reset();
        init950(fxtssbStoreBo);
        init1000(fxtssbStoreBo);
        return fxtssbStoreBo;
    }

    /**
     * source:fxtssb.0001.script
     */
    public void init1000(FxtssbStoreBo bo) {

        bo.getFxdgrp().getCbs().getNom1().setCur(xrtmodService.sysiso());
        // MAXAMT = 200000.00
//        Systems.setValues(fxtssbStoreVo.getFxdgrp().getRec(), "fxtyp", "SB");
        //结售汇类型  SB:即期结汇
        bo.getFxdgrp().getRec().setFxtyp("SB");
        String extkey = SettleContext.getUserSession().getUsr().getExtkey();
        bo.getFxdgrp().getRec().setUsr(extkey);
        //钞汇 默认 1:现汇
        bo.getFxtp().setCshflg("1");
        //这笔结售汇业务的成交日期，系统默认为当前日期，可以根据业务实际情况修改。
        bo.getFxdgrp().getRec().setOpndat(Dates.today());
        //这笔结售汇业务的起息日，系统默认为成交日，可以根据业务实际情况修改。
        bo.getFxdgrp().getRec().setValdat(bo.getFxdgrp().getRec().getOpndat());

    }

    private void init950(FxtssbStoreBo fxtssbStoreBo) {
    }

    @Override
    public void beforeSave(FxtssbStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1", "opn1", "opn2");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtssbStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getFxdgrp().getRec().getInr());
    }


    /**
     * source:fxtssb.0003.script
     */
    public void cbsSav(FxtssbStoreBo transactionVo, TrnmodVo trnmodVo) {
        cbsmodService.cbsTrnSetNew("TRN", trnmodVo.getTrn().getInr());
        Fxacbs cbs = transactionVo.getFxdgrp().getCbs();
        String trninr = transactionHelpService.getTrninr();
        trnmodService.cbsTrnStore("MAXAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp(), trninr);
        if (!MdaUtils.isEmpty(cbs.getNom1().getAmt())) {
            trnmodService.cbsTrnStore("NOMAMT", "AMT1", Dates.today(), cbs.getNom1().getCur(), cbs.getNom1().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp(), trninr);
            trnmodService.cbsTrnStore("UTLAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp(), trninr);
        }
    }

    /**
     * source:fxtssb.0022.script
     */
    public Map<String, List<String>> defaultAccountsN1000(FxtssbStoreVo fxtssbStoreVo) {
        fxtssbStoreVo.reset();
        String trnTyp = NULLSTR;
        String accLst = NULLSTR;
        IStream accStm = new StreamImpl();
        String acc2Lst = NULLSTR;
        IStream acc2Stm = new StreamImpl();
        Map<String, List<String>> map = new HashMap();
        if (!MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getCbs().getMax().getCur()) && !MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getApl().getPts().getPtyinr())) {
            if (!MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getRec().getDsp())) {
                trnTyp = fxtssbStoreVo.getFxdgrp().getRec().getDsp();
                if (MdaUtils.compareTo(trnTyp, "CA") == 0) {
                    trnTyp = "LO-TT2";
                }
                if (MdaUtils.compareTo(trnTyp, "SP") == 0 || MdaUtils.compareTo(trnTyp, "HA") == 0) {
                    accLst = glemodService.getAvailAcc(trnTyp, fxtssbStoreVo.getFxdgrp().getCbs().getMax().getCur(), "", fxtssbStoreVo.getFxdgrp().getCbs().getMax().getAmt());
                } else {
                    accLst = glemodService.getAvailAcc(trnTyp, fxtssbStoreVo.getFxdgrp().getCbs().getMax().getCur(), fxtssbStoreVo.getFxdgrp().getApl().getPts().getPtyinr(), fxtssbStoreVo.getFxdgrp().getCbs().getMax().getAmt());
                }
//                Systems.setValues(fxtssbStoreVo.getFxdgrp().getRec(), "acc", accLst);
                if (StringUtils.isNotEmpty(accLst)) {
                    String[] split = accLst.split("[\\n]|\\r\\n");
                    map.put("accLst", new ArrayList<>(Arrays.asList(split)));
                }
                if (!fxtssbStoreVo.getFxdgrp().getRec().isModified("acc")) {
                    if (!MdaUtils.isEmpty(accLst)) {
                        Streams.streamClear(accStm);
                        Streams.streamSet(accStm, accLst);
                        fxtssbStoreVo.getFxdgrp().getRec().setAcc(Strings.getLine(accStm, 1));
                    }
                }
//                Systems.setEnabled(fxtssbStoreVo.getFxdgrp().getRec(), "acc");
            } else {
//                Dynamic.disableAndClearField(MdaDriver.getDatafield(fxtssbStoreVo.getFxdgrp().getRec(), "acc"));
                fxtssbStoreVo.getFxdgrp().getRec().setAcc(NULLSTR);
            }
            if (!MdaUtils.isEmpty(fxtssbStoreVo.getFxdgrp().getRec().getDsp2())) {
                if (MdaUtils.compareTo(fxtssbStoreVo.getFxdgrp().getRec().getDsp2(), "SP") == 0) {
                    acc2Lst = glemodService.getAvailAcc(fxtssbStoreVo.getFxdgrp().getRec().getDsp2(), "CNY", "", BigDecimal.ONE);
                } else {
                    acc2Lst = glemodService.getAvailAcc(fxtssbStoreVo.getFxdgrp().getRec().getDsp2(), "CNY", fxtssbStoreVo.getFxdgrp().getApl().getPts().getPtyinr(), BigDecimal.ONE);
                }
//                Systems.setValues(fxtssbStoreVo.getFxdgrp().getRec(), "acc2", acc2Lst);
                if (StringUtils.isNotEmpty(acc2Lst)) {
                    String[] split = acc2Lst.split("[\\n]|\\r\\n");
                    map.put("acc2Lst", new ArrayList<>(Arrays.asList(split)));
                }
                if (!fxtssbStoreVo.getFxdgrp().getRec().isModified("acc2")) {
                    if (!MdaUtils.isEmpty(acc2Lst)) {
                        Streams.streamClear(acc2Stm);
                        Streams.streamSet(acc2Stm, acc2Lst);
                        fxtssbStoreVo.getFxdgrp().getRec().setAcc2(Strings.getLine(acc2Stm, 1));
                    }
                }
//                Systems.setEnabled(fxtssbStoreVo.getFxdgrp().getRec(), "acc2");
            } else {
//                Dynamic.disableAndClearField(MdaDriver.getDatafield(fxtssbStoreVo.getFxdgrp().getRec(), "acc2"));
                fxtssbStoreVo.getFxdgrp().getRec().setAcc2(NULLSTR);
            }
        }
        return map;
    }

    public String getRef(FxtssbStoreVo fxtssbStoreVo) {
        fxtssbStoreVo.reset();
        PtsptaVo ptsptaVo = fxtssbStoreVo.getFxdgrp().getApl();
        if (ptsptaVo == null || StringUtils.isEmpty(ptsptaVo.getPtainr())) {
            return NULLSTR;
        }
        Pta pta = ptaCacheService.getPtaByInr(ptsptaVo.getPtainr());
        Bch wrkbch = new Bch();
        wrkbch.setInr(pta.getBranchinr());
        wrkbch = bchService.selectByPrimaryKey(wrkbch);
        if (Objects.isNull(wrkbch)) {
            return NULLSTR;
        }
        SettleContext.getUserSession().setWrkbch(wrkbch);
        Fxdgrp fxdgrp = fxtssbStoreVo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        return fxdgrp.getRec().getOwnref();
    }
}
